# =============================================================================
# PACKAGE DEPENDENCIES
# =============================================================================

library(dplyr)      # manipulate data frames
library(genderizeR) # use genderize.io API
library(readr)      # read/write CSV
library(rvest)      # read/extract HTML
library(stringr)    # manipulate strings

# =============================================================================
# INTERMEDIARY DATASETS
# =============================================================================

dir.create("data", showWarnings = FALSE)

f_seminars <- "data/semined_seminars.csv"
f_speakers <- "data/semined_speakers.csv"
f_genders  <- "data/semined_genders.csv"

# =============================================================================
# GET SEMINARS
# =============================================================================

if (!file.exists(f_seminars)) {
  
  cat("[>] Collecting INED seminars...\n")
  
  # base URLs
  p <- c(
    str_c("https://statapp.site.ined.fr/fr/",
          c("les-rencontres-passees", "les-rendez-vous")
    ),
    str_c("https://russ.site.ined.fr/fr/",
          c("seances-passees", "annee-2016-2017")
    )
  )
  
  # initialize master dataset
  d <- data_frame()
  
  for (i in p) {
    
    # extract all relevant nodesets (links to seminars)
    h <- read_html(i) %>% 
      html_nodes(".rte a")
    
    # temporary dataset
    h <- data_frame(
      title = html_text(h),
      url = html_attr(h, "href"),
      year = html_node(h, xpath = "..") %>% #  move up to seminar description
        html_text %>%
        str_extract("\\d{4}") # extract the year
    )
    
    # fix incomplete URLs and subset to relevant rows
    h <- mutate(h, url = str_replace(url, "^//", "https://")) %>% 
      filter(str_detect(url, "^http"))
    
    # impute missing years from URL year (fixes all missing cases)
    h$year[ is.na(h$year) ] <- str_extract(h$url[ is.na(h$year) ], "\\d{4}")
    
    # communicate on number of seminars
    cat("[+]", str_pad(nrow(h), 2), "URL(s) from", i, "\n")
    
    # bind to master dataset
    d <- rbind(d, cbind(seminar = str_extract(h$url, "russ|statapp"), h))
    
  }
  
  # clean up titles
  d$title <- str_replace_all(d$title, "\"|Séance\\s\\d+\\s?:\\s|\\s+L$", "")
  
  # communicate total number of seminars
  cat("[>]", str_pad(nrow(d), 2), "URLs saved\n")
  
  # save
  write_csv(d, f_seminars)
  
}

d <- read_csv(f_seminars, col_types = "cccc")
cat("[>]", str_pad(nrow(d), 2), "seminars loaded\n")

# =============================================================================
# GET SPEAKERS
# =============================================================================

if (!file.exists(f_speakers)) {
  
  cat("[>] Collecting INED seminar speakers...\n")
  
  # initialize master dataset
  p <- data_frame()
  
  for (i in d$url) {
    
    h <- try(
      read_html(i) %>% 
        html_nodes("#main strong") %>% 
        html_text,
      silent = TRUE
    )
    
    # skip missing pages
    if ("try-error" %in% class(h)) {
      
      cat("[!] ERROR at", i, "\n")
      next
      
    }
    
    # filter speakers
    h <- h[ str_detect(h, "^[A-Z]") &
              !str_detect(h, "\\d|Le\\sLundi|inscription|^Ined$") ]

    # skip missing data
    if (!length(h)) {
      
      cat("[!] NO SPEAKERS found at", i, "\n")
      next
      
    }
    
    # communicate on number of seminars
    cat("[+]", str_pad(length(h), 2), "speakers(s) from", i, "\n")
    
    # bind to master dataset
    p <- rbind(p, data_frame(url = i, speaker = h))
    
  }
  
  # clean up speakers
  p$speaker <- str_to_title(p$speaker) %>% 
    str_replace("’", "'") %>% 
    str_replace("\\sD'", " d'") %>% 
    str_replace("\\sDe\\s", " de ")
  
  # n = 1 special problems
  p$speaker[ p$speaker %in% c("Arnaud", "Bringé") ] <- "Arnaud Bringé"
  p$speaker <- str_replace(p$speaker, "^Chistophe", "Christophe")
  
  # remove duplicated speakers
  p <- unique(p)
  
  # communicate total number of speakers
  cat("[>]", str_pad(nrow(p), 2), "speakers(s) saved\n")
  
  # save
  write_csv(p, f_speakers)
  
}

p <- read_csv(f_speakers, col_types = "cc")
cat("[>]", str_pad(nrow(d), 2), "speakers loaded\n")

cat("[>] Dropping", n_distinct(d$url) - n_distinct(p$url),
    "seminars with no speakers\n")

d <- inner_join(d, p, by = "url")
# View(d)

# =============================================================================
# GET GENDERS
# =============================================================================

# extract first names
n <- str_replace(d$speaker, "(.*?)\\s(.*)", "\\1") %>% 
  unique

if (!file.exists(f_genders)) {
  
  cat("[>] Guessing", length(n), "genders...\n")
  g <- findGivenNames(n, textPrepare = FALSE, progress = TRUE)
  
  # View(n)
  write_csv(g, f_genders)

}

g <- read_csv(f_genders, col_types = "ccdd")

d$name = str_replace(d$speaker, "(.*?)\\s(.*)", "\\1")

cat("[>] Dropping", sum(!d$name %in% g$name), "ungenderized names\n")

# =============================================================================
# PROPORTION OF WOMEN SPEAKERS
# =============================================================================

d <- inner_join(d, g, by = "name") %>% 
  select(-name, -count)

print(t <- table(d$seminar, d$gender))
# print(prop.table(t, 1))
# print(prop.test(t))

saveRDS(t, file = "semined.rds")

# kthxbye
